/*
    File Prism.cpp

    Fancy diamond cut: Prism.

    Copyright (c) 2006 by OctoNus Ltd.
    All rights reserved.
*/

#define DC_PARAMETERS_VERSION 2

#include "windows.h"
#include "math.h"

#include "DiamCut.h"


// Unique identifiers

// {{74C5E67E-C46B-4DC9-84B3-C891AE6C6D2E}}
static const CLSID CLSID_Prism = { 0x74C5E67E, 0xC46B, 0x4DC9, { 0x84, 0xB3, 0xC8, 0x91, 0xAE, 0x6C, 0x6D, 0x2E } };


// Definition of data structure of cut parameters
// The structure does not depend on the number of points on the girdle,
// i.e. it is the same for tria64 and tria32
struct PrismData
{
		int		nVersion;		// data version
        // Main parameter
		double	dblDiameter;	// diameter
		// Parameters
		double	dblGirdleThickness;		// Girdle height
		double	dblRt;		// Girdle Ratio
};

// Exported functions
//
// int PrismGetCuttingList (int * cuts, int n);
// int PrismGetCuttingInfo (void * data, int cut, int flavor);
// int PrismInitDiamond (DiamondCutting * diam, int cut);
// int PrismInitVertex (DiamondCutting * diam, int cut);
// int PrismIsCorrect (DiamondCutting * diam, int cut);
// int PrismGetParameter (double * value, int * param, int nparams, DiamondCutting * diam, int cut);
// int PrismSetParameter (double * value, int * param, int nparams, DiamondCutting * diam, int cut);

// Functions that are used internally
// Getting / setting parameters
static int PrismGetSingleParameter (double & dblValue, int nParam, PrismData * pData);
static int PrismSetSingleParameter (double dblValue, int nParam, PrismData * pData);

// Internal numeration of cuts in this module (library)
enum
{
        Prism = 1,
};


// Vertex structure

static const PHINDEX IndexOfPrism [ ] =
{
	// Table shoud be the first side in list
	0, 1, 2, 3, 0,
	0, 4, 5, 1, 0,
	1, 5, 6, 2, 1,
	2, 6, 7, 3, 2,
	3, 7, 4, 0, 3,
	4, 7, 6, 5, 4,
	// ending flag
	MAXPHINDEX
};


#define DEGREE          0.01745329251994
#define PERCENT         0.01

// Set of default parameters
static  PrismData PrismPars =
{
	1,			// version
    // Main parameter
	6.0,		// Diameter
	3.5 * PERCENT,		// TODO. Initialization for dblGirdleThickness
	1.0 		// TODO. Initialization for dblRt
};


// Implementation of the interface of exported cuts


// Exported function: List of implemented cuts.
//      The function fills the 'cuts' list with cuts identifiers
// that are unique for this module.
// Returned value: number of cuts
// External name: GetCuttingList
// External name is different from the internal one to allow
//   direct linking of this module sometimes, without use of
//   LoadLibrary.Windows API
int PrismGetCuttingList (int * pnCuts, int nLength)
{
	int i = 0;
	if (i < nLength) pnCuts [i++] = Prism;
	return i;
}

#ifdef _DEBUG
// Example for output debug string
extern "C"
{
static int nullPrintf (const char *, ...) { return 0; }
static PrintfFuncType PrintfFunc = nullPrintf;
};
#endif


// Exported function: getting misc. information on the cut
// External name: GetCuttingInfo
// Returned value: 1, if the requested information is returned
int PrismGetCuttingInfo (void * pData, int nCut, int nFlavor)
{
	switch (nFlavor)
	{
		default: return 0;
		case DC_ClassDiamondCutting: // cutting class
        {
            ClassDiamondCutting * pInfo = (ClassDiamondCutting*) pData;
            switch (nCut)
            {
                default: return 0;
                case Prism :
					pInfo->dataSize = sizeof(PrismData);
					pInfo->uid = CLSID_Prism;
					pInfo->version = 1;
					pInfo->sides = IndexOfPrism;
					pInfo->name = "Prism";
					pInfo->shortname = "Prism";
					break;
			}
			break;
        }
        case DC_ParameterList: // parameter lists
        {
            ParameterList * pList = (ParameterList *) pData;
			if (nCut != Prism) return 0;
			switch (pList->listType)
            {
                default: return 0;
                case DC_FullParameterList: // full parameter list
				{
					if (pList->count < pList->bufSize) pList->pars [pList->count++] = DC_Diameter;
					if (pList->count < pList->bufSize) pList->pars [pList->count++] = DC_GirdleThickness;
					if (pList->count < pList->bufSize) pList->pars [pList->count++] = DC_GirdleRatio;
				}
				break;
            }
            break;
        }
        case DC_ParameterInfo: // information on the specific parameter
        {
            ParameterInfo * pInfo = (ParameterInfo *) pData;
			if (nCut != Prism) return 0;

			break;
        }
#ifdef _DEBUG
        case DC_OutputStringInterface:
        {
            PrintfFunc = (PrintfFuncType) pData;
            break;
        }
#endif
    }
    return 1;
}

// Exported initialization of cut data
// External name: InitDiamond.
// Returned value: 1, if the initialization completed successfully
int PrismInitDiamond (DiamondCutting * pDiam, int nCut)
{
    switch (nCut)
    {
        default: return 0;
        case Prism :
        {
            // Default initialization simply copies parameter values
            // from PrismPars structure
            PrismData * pData = (PrismData*) pDiam->data;
            * pData = PrismPars;
            break;
        }
    }
    return 1;
}


// Exported function of recalculating points' coordinates
// External name: InitVertex.
// Returned value: 1, if points are recalculated
int PrismInitVertex (DiamondCutting * pDiam, int nCut)
{
	if (nCut != Prism) return 0;
	POINT3 * p3Vertex = pDiam->vertex;
	PrismData & data = *(PrismData*) pDiam->data;


	// TODO. Use parameters for vertex initialization
	// dblGirdleThickness (DC_GirdleThickness)
	// dblRt (DC_GirdleRatio)
	double dblX = data.dblDiameter / 2;
	double dblY = data.dblDiameter * data.dblRt / 2;
	double dblZ = data.dblDiameter * data.dblGirdleThickness / 2;
	p3Vertex [0] = POINT3 (-dblX, dblY, dblZ);
	p3Vertex [1] = POINT3 (-dblX,-dblY, dblZ);
	p3Vertex [2] = POINT3 ( dblX,-dblY, dblZ);
	p3Vertex [3] = POINT3 ( dblX, dblY, dblZ);
	p3Vertex [4] = POINT3 (-dblX, dblY,-dblZ);
	p3Vertex [5] = POINT3 (-dblX,-dblY,-dblZ);
	p3Vertex [6] = POINT3 ( dblX,-dblY,-dblZ);
	p3Vertex [7] = POINT3 ( dblX, dblY,-dblZ);

    return 1;
}

// Exported function of correctness of points' recalculation
// External name: IsCorrect.
// The returned value: 1, if the parameter set is correct, i.e. this
// parameters can be used for point recalculation and the diamond will be usable
int PrismIsCorrect (DiamondCutting * pDiam, int nCut)
{
	if (nCut != Prism) return 0;
	PrismData & data = *(PrismData*) pDiam->data;
	// Simple check of parameters
	if (! (data.nVersion <= 1 &&
		// TODO. Use parameters for checking correctness of diamond
		// dblGirdleThickness (DC_GirdleThickness)
		// dblRt (DC_GirdleRatio)

		0 < data.dblRt &&
		0 < data.dblGirdleThickness &&
		data.dblDiameter > 1e-3)) return 0;
	return 1;
}

// Exported function of getting diamond parameter
// External name: GetParameter.
// Returned value: 1, if the parameter set is received successfully
int PrismGetParameter (double * pDblValue, int * pnParam, int nParams,
	DiamondCutting * pDiam, int nCut)
{
	if (nCut != Prism) return 0;
	PrismData & data = *(PrismData*) pDiam->data;
	for (int i = 0; i < nParams; i++) if (! PrismGetSingleParameter (pDblValue [i], pnParam [i], & data)) return 0;
    return 1;
}

// Exported function of setting diamond parameter
// External name: SetParameter.
// Returned value: 1, if the parameter set is applied
// Unfortunately, setting a set of parameters is implemented by
// setting parameters one-by-one!
int PrismSetParameter (double * pDblValue, int * pnParam, int nParams,
	DiamondCutting * pDiam, int nCut)
{
	if (nCut != Prism) return 0;
	PrismData & data = *(PrismData*) pDiam->data;
	for (int i = 0; i < nParams; i++) if (! PrismSetSingleParameter (pDblValue [i], pnParam [i], & data)) return 0;
    return 1;
}


// Getting single parameter
static int PrismGetSingleParameter (double & dblValue, int nParam, PrismData * pData)
{
	switch (nParam)
	{
		default: return 0;
        case DC_Diameter: dblValue = pData->dblDiameter; break;
		case DC_GirdleThickness: dblValue = pData->dblGirdleThickness; break;
		case DC_GirdleRatio: dblValue = pData->dblRt; break;

	}
	return 1;
}

// Setting single parameter
static int PrismSetSingleParameter (double dblValue, int nParam, PrismData * pData)
{
	switch (nParam)
	{
		default: return 0;
        case DC_Diameter: pData->dblDiameter = dblValue; break;
		case DC_GirdleThickness: pData->dblGirdleThickness = dblValue; break;
		case DC_GirdleRatio: pData->dblRt = dblValue; break;

	}
	return 1;
}

